"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = runDev;
Object.defineProperty(exports, "fork", {
  enumerable: true,
  get: function get() {
    return _fork2.default;
  }
});

var _dev = _interopRequireDefault(require("af-webpack/dev"));

var _chalk = _interopRequireDefault(require("chalk"));

var _umiPlugin = require("umi-plugin");

var _registerBabel = _interopRequireDefault(require("./registerBabel"));

var _getWebpackConfig = _interopRequireDefault(require("./getWebpackConfig"));

var _createRouteMiddleware = _interopRequireDefault(require("./createRouteMiddleware"));

var _generateEntry = _interopRequireWildcard(require("./generateEntry"));

var _send = _interopRequireWildcard(require("./send"));

var _getConfig = require("./getConfig");

var _watch = require("./getConfig/watch");

var _getPaths = _interopRequireDefault(require("./getPaths"));

var _fork2 = _interopRequireDefault(require("af-webpack/lib/fork"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('umi-build-dev:dev');

function runDev(opts) {
  const _opts$cwd = opts.cwd,
        cwd = _opts$cwd === void 0 ? process.cwd() : _opts$cwd,
        babel = opts.babel,
        disableCSSModules = opts.disableCSSModules,
        extraResolveModules = opts.extraResolveModules,
        _opts$libraryName = opts.libraryName,
        libraryName = _opts$libraryName === void 0 ? 'umi' : _opts$libraryName,
        _opts$staticDirectory = opts.staticDirectory,
        staticDirectory = _opts$staticDirectory === void 0 ? 'static' : _opts$staticDirectory,
        _opts$tmpDirectory = opts.tmpDirectory,
        tmpDirectory = _opts$tmpDirectory === void 0 ? `.${libraryName}` : _opts$tmpDirectory,
        _opts$outputPath = opts.outputPath,
        outputPath = _opts$outputPath === void 0 ? './dist' : _opts$outputPath,
        pluginFiles = opts.plugins,
        preact = opts.preact,
        _opts$extraMiddleware = opts.extraMiddlewares,
        extraMiddlewares = _opts$extraMiddleware === void 0 ? [] : _opts$extraMiddleware;
  const plugins = (0, _umiPlugin.resolvePlugins)(pluginFiles);
  const paths = (0, _getPaths.default)({
    cwd,
    tmpDirectory,
    outputPath
  }); // 为配置注册 babel 解析

  (0, _registerBabel.default)(babel, {
    configOnly: true
  }); // 获取用户配置

  let config = null;
  let watchConfig = null;

  try {
    const configObj = (0, _getConfig.getConfig)(cwd, {
      force: true
    });
    config = configObj.config;
    watchConfig = configObj.watch;
  } catch (e) {
    console.error(_chalk.default.red(e.message));
    debug('get config failed, watch config and reload'); // 监听配置项变更，然后重新执行 dev 逻辑

    (0, _getConfig.watchConfigs)().on('all', (event, path) => {
      debug(`[${event}] ${path}, unwatch and reload`); // 重新执行 dev 逻辑

      (0, _watch.unwatch)();
      runDev(opts);
    });
    return;
  } // 生成入口文件


  let watchEntry = null;

  try {
    debug(`libraryName: ${libraryName}`);
    const entryGObj = (0, _generateEntry.default)({
      cwd,
      plugins,
      routerTpl: opts.routerTpl,
      entryJSTpl: opts.entryJSTpl,
      libraryName,

      onChange(routeConfig) {
        sendPageList(routeConfig);
      },

      paths
    });
    watchEntry = entryGObj.watch;
  } catch (e) {
    console.error(_chalk.default.red(e.message));
    console.error(_chalk.default.red(e.stack));
    debug('generate entry failed, watch pages and reload');
    (0, _generateEntry.watchPages)({
      cwd,
      paths,

      onChange(watcher) {
        watcher.close();
        runDev(opts);
      }

    });
    return;
  } // 获取 webpack 配置


  const webpackConfig = (0, _getWebpackConfig.default)({
    cwd,
    config,
    babel,
    disableCSSModules,
    extraResolveModules,
    libraryName,
    staticDirectory,
    paths,
    preact
  });
  debug(`webpackConfig: ${JSON.stringify(webpackConfig)}`); // af-webpack dev

  (0, _dev.default)({
    webpackConfig,
    extraMiddlewares: [(0, _createRouteMiddleware.default)(cwd, config, plugins, staticDirectory, libraryName, paths), ...extraMiddlewares],

    afterServer(devServer) {
      // 监听配置变更
      debug('watch configs');
      watchConfig(devServer);
      watchEntry(devServer);
    }

  });
}

function sendPageList(routeConfig) {
  const pageList = Object.keys(routeConfig || {}).map(path => ({
    path
  }));
  (0, _send.default)({
    type: _send.PAGE_LIST,
    payload: pageList
  });
} // 给 koi-core 用，后期需要干掉，fork 的功能集中在 af-webpack 里完成