"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path = require("path");

var _fs = require("fs");

var _getConfig = _interopRequireDefault(require("af-webpack/getConfig"));

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _browsers = _interopRequireDefault(require("./defaultConfigs/browsers"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const debug = require('debug')('umi-build-dev:getWebpackConfig');

const env = process.env.NODE_ENV;

function _default(opts = {}) {
  const cwd = opts.cwd,
        config = opts.config,
        babel = opts.babel,
        hash = opts.hash,
        disableCSSModules = opts.disableCSSModules,
        routeConfig = opts.routeConfig,
        libraryName = opts.libraryName,
        staticDirectory = opts.staticDirectory,
        extraResolveModules = opts.extraResolveModules,
        paths = opts.paths,
        preact = opts.preact; // browsers 配置同时给 babel-preset-env 和 autoprefixer 用

  const browsers = config.browsers || _browsers.default; // entry

  const entryScript = (0, _path.join)(cwd, `./${paths.tmpDirPath}/${libraryName}.js`);
  const setPublicPathFile = (0, _path.join)(__dirname, '../template/setPublicPath.js');
  const isDev = env === 'development';
  const entry = isDev ? {
    [libraryName]: [_reactDevUtils.webpackHotDevClientPath, entryScript]
  } : {
    [libraryName]: [setPublicPathFile, entryScript]
  };
  const pageCount = isDev ? null : Object.keys(routeConfig).length;
  debug(`pageCount: ${pageCount}`);
  debug(`config: ${JSON.stringify(config)}`); // default react, support config with preact

  const reactAlias = preact ? {
    react: require.resolve('preact-compat'),
    'react-dom': require.resolve('preact-compat'),
    'create-react-class': require.resolve('preact-compat/lib/create-react-class')
  } : {
    react: require.resolve('react'),
    'react-dom': require.resolve('react-dom')
  }; // 关于为啥放 webpack 而不放 babel-plugin-module-resolver 里
  // 详见：https://tinyletter.com/sorrycc/letters/babel

  const libAlias = {
    'antd-mobile': (0, _path.dirname)(require.resolve('antd-mobile/package')),
    antd: (0, _path.dirname)(require.resolve('antd/package'))
  }; // 支持用户指定 antd 和 antd-mobile 的版本
  // TODO: 出错处理，用户可能指定了依赖，但未指定 npm install

  const pkgPath = (0, _path.join)(cwd, 'package.json');

  if ((0, _fs.existsSync)(pkgPath)) {
    const _require = require(pkgPath),
          _require$dependencies = _require.dependencies,
          dependencies = _require$dependencies === void 0 ? {} : _require$dependencies;

    if (dependencies.antd) {
      libAlias['antd'] = (0, _path.dirname)(require.resolve((0, _path.join)(cwd, 'node_modules/antd/package')));
    }

    if (dependencies['antd-mobile']) {
      libAlias['antd-mobile'] = (0, _path.dirname)(require.resolve((0, _path.join)(cwd, 'node_modules/antd-mobile/package')));
    }
  }

  return (0, _getConfig.default)(_extends({
    cwd,
    entry,
    autoprefixer: {
      browsers
    },
    babel: {
      presets: [[babel, {
        browsers
      }]]
    },
    theme: config.theme,
    outputPath: (0, _path.join)(paths.absOutputPath, staticDirectory),
    hash: !isDev && hash,
    disableCSSModules,
    extraResolveModules,
    define: _extends({
      // 禁用 antd-mobile 升级提醒
      // ref: http://gitlab.alipay-inc.com/twa/koi-pkgs/issues/53
      'process.env.DISABLE_ANTD_MOBILE_UPGRADE': true
    }, config.define || {}),
    alias: _extends({}, reactAlias, libAlias, config.alias || {})
  }, isDev ? {
    // 生成环境的 publicPath 是服务端把 assets 发布到 cdn 后配到 HTML 里的
    // 开发环境的 publicPath 写死 /static/
    publicPath: `/${staticDirectory}/`
  } : {}, {
    commons: isDev ? [] : [{
      async: '__common',
      children: true,

      minChunks(module, count) {
        if (pageCount <= 2) {
          return count >= pageCount;
        }

        return count >= pageCount * 0.5;
      }

    }]
  }));
}

module.exports = exports["default"];